declare const styles: {
  'root': string
  'vhidden': string
  'button_reset': string
  'button': string
  'day_selected': string
  'months': string
  'month': string
  'table': string
  'with_weeknumber': string
  'caption': string
  'multiple_months': string
  'caption_dropdowns': string
  'caption_label': string
  'nav': string
  'caption_start': string
  'caption_end': string
  'nav_button': string
  'dropdown_year': string
  'dropdown_month': string
  'dropdown': string
  'dropdown_icon': string
  'head': string
  'head_row': string
  'row': string
  'head_cell': string
  'tbody': string
  'tfoot': string
  'cell': string
  'weeknumber': string
  'day': string
  'day_today': string
  'day_outside': string
  'day_range_start': string
  'day_range_end': string
  'day_range_middle': string
}

export default styles
