declare const styles: {
  'rdp': string
  'rdp-vhidden': string
  'rdp-button_reset': string
  'rdp-button': string
  'rdp-day_selected': string
  'rdp-months': string
  'rdp-month': string
  'rdp-table': string
  'rdp-with_weeknumber': string
  'rdp-caption': string
  'rdp-multiple_months': string
  'rdp-caption_dropdowns': string
  'rdp-caption_label': string
  'rdp-nav': string
  'rdp-caption_start': string
  'rdp-caption_end': string
  'rdp-nav_button': string
  'rdp-dropdown_year': string
  'rdp-dropdown_month': string
  'rdp-dropdown': string
  'rdp-dropdown_icon': string
  'rdp-head': string
  'rdp-head_row': string
  'rdp-row': string
  'rdp-head_cell': string
  'rdp-tbody': string
  'rdp-tfoot': string
  'rdp-cell': string
  'rdp-weeknumber': string
  'rdp-day': string
  'rdp-day_today': string
  'rdp-day_outside': string
  'rdp-day_range_start': string
  'rdp-day_range_end': string
  'rdp-day_range_middle': string
}

export default styles
